#coding: utf-8

import ConfigParser
import importlib
import time
from Queue import Empty
from multiprocessing import Process, Queue
import os
import pystatsd
import sys
import socket

from log import logger, setup_logging
from optparse import OptionParser


def start_monitors(feed_queue):
    '''
    Запускаем подпроцессы для каждого из модулей мониторинга.
    '''
    result = {}
    modules_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'monitors')
    
    modules = [fn for fn in os.listdir(modules_path) if fn.endswith('.py')]

    logger.debug('Attempting to start all monitors.')
    processes = []
    for fn in modules:
        if fn == '__init__.py':
            continue

        name, ext = os.path.splitext(fn)
        module = importlib.import_module('.monitors.{0}'.format(name), package=__package__)
        logger.debug('Starting "{0}"...'.format(name))

        proc = Process(target=module.collect, args=(feed_queue,))
        proc.start()
    return processes


def main():
    setup_logging()

    cfg_files = ['./tipster.cfg']
    parser = OptionParser()
    parser.add_option('-c', '--cfg', dest='cfg_file',
                      help='Load additional config file.', metavar='<CONFIG>')
    parser.add_option('-d', '--dump', dest='just_dump', action='store_true',
                      help='Just dump results to stdout.')
    (options, args) = parser.parse_args()

    if options.cfg_file:
        cfg_files.append(options.cfg_file)

    config = ConfigParser.SafeConfigParser()
    if not config.read(cfg_files):
        logger.error('None of configuration files were parsed: {0}'.format(', '.join(cfg_files)))
        return        

    client = config.get('identity', 'client')
    region = config.get('identity', 'region')
    product = config.get('identity', 'product')
    try:
        instance = config.get('identity', 'instance')
    except ConfigParser.NoOptionError:
        instance = None

    try:
        host = config.get('statsd', 'host')        
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) as e:
        host = 'localhost'

    try:
        port = config.getint('statsd', 'port')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) as e:
        port = 8125

    statsd = pystatsd.Client(host, port)
    metric_handlers = {
        'gauge': statsd.gauge,
        'incr': statsd.increment,
        'decr': statsd.decrement,
        'timing': statsd.timing
    }

    # Даже если эти значения есть в конфиге, они все должны содержать в себе
    # какие-либо данные.
    if not all([client, region, product]):
        logger.error('You must fill out [identity] section in config file!')
        return

    if instance:
        identity_prefix = '{0}.{1}.{2}.instance_{3}'.format(region, client, product, instance)
    else:
        identity_prefix = '{0}.{1}.{2}'.format(region, client, product)

    q = Queue()
    monitors = start_monitors(q)

    logger.info('Our identity prefix: {0}'.format(identity_prefix))

    while True:        
        try:
            metric, value, m_type = q.get(True, 10)
            assert m_type in metric_handlers, 'Unknown metric type: "{0}"'.format(m_type)

            logger.debug('Received data: {0} {1} {2}'.format(metric, value, m_type))
            
            handler = metric_handlers[m_type]
            handler(''.join(['v1', identity_prefix, metric]), value)
        except Empty:
            pass
        except KeyboardInterrupt:
            print '\nCtrl-C, Ctrl-V.'
            break

    logger.info('Shutting down...')

if __name__ == '__main__':
    main()